@@ -39,6 +39,7 @@
 
 #include "config.h"
 
+static PayloadType *payload_type_clearmode;
 static PayloadType *payload_type_efr;
 static PayloadType *payload_type_hr;
 static RtpProfile *osmo_pt_profile;
@@ -46,7 +47,7 @@ static RtpProfile *osmo_pt_profile;
 static void *tall_rtp_ctx;
 
 /* malloc integration */
-
+#if HAVE_ORTP_MEM_FUNC
 static void *osmo_ortp_malloc(size_t sz)
 {
 	return talloc_size(tall_rtp_ctx, sz);
@@ -67,6 +68,7 @@ static OrtpMemoryFunctions osmo_ortp_memfn = {
 	.realloc_fun = osmo_ortp_realloc,
 	.free_fun = osmo_ortp_free
 };
+#endif /* HAVE_ORTP_MEM_FUNC */
 
 /* logging */
 
@@ -120,13 +122,14 @@ static void my_ortp_logfn(
 
 	needs_endl = fmt[fmt_len - 1] != '\n' ? 1 : 0;
 
-	str = osmo_ortp_malloc(domain_len + 2 /*": "*/ + fmt_len + needs_endl + 1);
-	sprintf(str, "%s%s%s%s", domain_str, domain_len ? ": " : "", fmt, needs_endl ? "\n" : "");
+	str = talloc_asprintf(tall_rtp_ctx, "%s%s%s%s",
+			      domain_str, domain_len ? ": " : "",
+			      fmt, needs_endl ? "\n" : "");
 
 	osmo_vlogp(DLMIB, ortp_to_osmo_lvl(lev), __FILE__, 0,
 		   0, str, args);
 
-	osmo_ortp_free(str);
+	talloc_free(str);
 
 }
 
@@ -262,10 +265,20 @@ static int osmo_rtp_socket_fdreg(struct osmo_rtp_socket *rs)
 	return 0;
 }
 
-static void create_payload_types()
+static void create_payload_types(void)
 {
 	PayloadType *pt;
 
+	/* CLEARMODE as per RFC 4040 (chapter 3) */
+	pt = payload_type_new();
+	pt->type = PAYLOAD_OTHER;
+	pt->clock_rate = 8000;
+	pt->bits_per_sample = 8;
+	pt->mime_type = "CLEARMODE";
+	pt->normal_bitrate = 64000;
+	pt->channels = 1;
+	payload_type_clearmode = pt;
+
 	/* EFR */
 	pt = payload_type_new();
 	pt->type = PAYLOAD_AUDIO_PACKETIZED;
@@ -290,6 +303,7 @@ static void create_payload_types()
 	/* add the GSM specific payload types. They are all dynamically
 	 * assigned, but in the Osmocom GSM system we have allocated
 	 * them as follows: */
+	rtp_profile_set_payload(osmo_pt_profile, RTP_PT_CSDATA, payload_type_clearmode);
 	rtp_profile_set_payload(osmo_pt_profile, RTP_PT_GSM_EFR, payload_type_efr);
 	rtp_profile_set_payload(osmo_pt_profile, RTP_PT_GSM_HALF, payload_type_hr);
 	rtp_profile_set_payload(osmo_pt_profile, RTP_PT_AMR, &payload_type_amr);
@@ -303,7 +317,9 @@ static void create_payload_types()
 void osmo_rtp_init(void *ctx)
 {
 	tall_rtp_ctx = ctx;
+#if HAVE_ORTP_MEM_FUNC
 	ortp_set_memory_functions(&osmo_ortp_memfn);
+#endif
 	ortp_init();
 	ortp_set_log_level_mask(
 #if HAVE_ORTP_LOG_DOMAIN
