@@ -238,30 +238,34 @@ static int encode8_handle_ta(ubit_t *trau_bits, const struct osmo_trau_frame *fr
 }
 
 /* TS 08.60 Section 3.1.1 */
-static int encode16_fr(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
+static int encode16_fr(ubit_t *trau_bits, const struct osmo_trau_frame *fr, bool is_tfo)
 {
 	const ubit_t *cbits5;
 	int i;
 	int d_idx = 0;
 
-	switch (fr->type) {
-	case OSMO_TRAU16_FT_IDLE:
-		if (fr->dir == OSMO_TRAU_DIR_UL)
-			cbits5 = ft_idle_up_bits;
-		else
-			cbits5 = ft_idle_down_bits;
-		break;
-	case OSMO_TRAU16_FT_FR:
-		if (fr->dir == OSMO_TRAU_DIR_UL)
-			cbits5 = ft_fr_up_bits;
-		else
-			cbits5 = ft_fr_down_bits;
-		break;
-	case OSMO_TRAU16_FT_EFR:
-		cbits5 = ft_efr_bits;
-		break;
-	default:
-		return -EINVAL;
+	if (is_tfo) {
+		cbits5 = fr->c_bits;
+	} else {
+		switch (fr->type) {
+		case OSMO_TRAU16_FT_IDLE:
+			if (fr->dir == OSMO_TRAU_DIR_UL)
+				cbits5 = ft_idle_up_bits;
+			else
+				cbits5 = ft_idle_down_bits;
+			break;
+		case OSMO_TRAU16_FT_FR:
+			if (fr->dir == OSMO_TRAU_DIR_UL)
+				cbits5 = ft_fr_up_bits;
+			else
+				cbits5 = ft_fr_down_bits;
+			break;
+		case OSMO_TRAU16_FT_EFR:
+			cbits5 = ft_efr_bits;
+			break;
+		default:
+			return -EINVAL;
+		}
 	}
 
 	encode_sync16(trau_bits);
@@ -314,17 +318,21 @@ static int decode16_fr(struct osmo_trau_frame *fr, const ubit_t *trau_bits,
 }
 
 /* TS 08.60 Section 3.1.2 */
-static int encode16_amr(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
+static int encode16_amr(ubit_t *trau_bits, const struct osmo_trau_frame *fr, bool is_tfo)
 {
-	const ubit_t *cbits5 = ft_amr_bits;
 	int i, d_idx;
 
 	encode_sync16(trau_bits);
 
-	/* C1 .. C5 */
-	memcpy(trau_bits + 17, cbits5 + 0, 5);
-	/* C6 .. C15 */
-	memcpy(trau_bits + 17 + 5, fr->c_bits + 5, 15 - 5);
+	if (is_tfo) {
+		/* C1 .. C15 */
+		memcpy(trau_bits + 17, fr->c_bits, 15);
+	} else {
+		/* C1 .. C5 */
+		memcpy(trau_bits + 17, ft_amr_bits, 5);
+		/* C6 .. C15 */
+		memcpy(trau_bits + 17 + 5, fr->c_bits + 5, 15 - 5);
+	}
 
 	trau_bits[32] = 1;
 	/* C16 .. C25 */
@@ -549,7 +557,7 @@ static ubit_t *trau_idle_frame(void)
 		fr_idle_frame.c_bits[11] = 1; /* C12 (UFE), good frame */
 		fr_idle_frame.c_bits[15] = 1; /* C16 (SP), no DTX */
 
-		encode16_fr(encoded_idle_frame, &fr_idle_frame);
+		encode16_fr(encoded_idle_frame, &fr_idle_frame, false);
 		dbits_initted = 1; /* set it to 1 to not call it again */
 	}
 	return encoded_idle_frame;
@@ -650,6 +658,22 @@ static int encode16_data(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	return 40 * 8;
 }
 
+/* TS 48.060 section 5.5.3 */
+static int encode16_d145_sync(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
+{
+	encode_sync16(trau_bits);
+
+	/* C1 .. C5 */
+	memcpy(trau_bits + 17, ft_d145s_bits, 5);
+	/* C6 .. C15 */
+	memcpy(trau_bits + 17 + 5, fr->c_bits + 5, 15 - 5);
+
+	/* This frame is for sync only, all data bits filled with 1s */
+	memset(trau_bits + 4 * 8, 1, (40 - 4) * 8);
+
+	return 40 * 8;
+}
+
 /* TS 08.60 3.3.2 */
 static int decode16_edata(struct osmo_trau_frame *fr, const ubit_t *trau_bits,
 			  enum osmo_trau_frame_direction dir)
@@ -676,6 +700,10 @@ static int encode16_edata(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	else
 		cbits5 = ft_edata_bits;
 
+	/* sync pattern */
+	memset(trau_bits, 0, 16);
+	trau_bits[16] = 1;
+
 	/* C1 .. C5 */
 	memcpy(trau_bits + 17, cbits5, 5);
 
@@ -823,7 +851,7 @@ static ubit_t compute_odd_parity(const ubit_t *in, unsigned int num_bits)
 }
 
 /* TS 08.61 Section 5.2.1.1 */
-static int encode8_hr(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
+static int encode8_hr(ubit_t *trau_bits, const struct osmo_trau_frame *fr, bool is_tfo)
 {
 	int i, d_idx = 0;
 
@@ -838,11 +866,15 @@ static int encode8_hr(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	/* C1 .. C5 */
 	ubit_t *cbits_out = trau_bits + 1 * 8 + 1;
 	if (fr->dir == OSMO_TRAU_DIR_UL) {
-		cbits_out[0] = 0;
-		cbits_out[1] = 0;
-		cbits_out[2] = 0;
-		cbits_out[3] = 1;
-		cbits_out[4] = 0;
+		if (is_tfo) {
+			memcpy(cbits_out, fr->c_bits, 5);
+		} else {
+			cbits_out[0] = 0;
+			cbits_out[1] = 0;
+			cbits_out[2] = 0;
+			cbits_out[3] = 1;
+			cbits_out[4] = 0;
+		}
 	} else {
 		cbits_out[0] = 0;
 		cbits_out[1] = 0;
@@ -1110,12 +1142,13 @@ static int decode8_data(struct osmo_trau_frame *fr, const ubit_t *trau_bits,
 	d_idx += 2;
 	/* D3 .. D8 */
 	memcpy(fr->d_bits + d_idx, trau_bits + 2 * 8 + 2, 6);
-	/* D9 .. D57 + D'1 .. D'57 */
-	for (i = 3; i < 20; i++) {
+	d_idx += 6;
+	/* D9 .. D63 + D'1 .. D'57 */
+	for (i = 3; i < 19; i++) {
 		memcpy(fr->d_bits + d_idx, trau_bits + i * 8 + 1, 7);
 		d_idx += 7;
 	}
-	/* D'58 .. D'62 */
+	/* D'58 .. D'63 */
 	memcpy(fr->d_bits + d_idx, trau_bits + 19 * 8 + 1, 6);
 	d_idx += 6;
 
@@ -1132,7 +1165,7 @@ static int encode8_data(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	trau_bits[1 * 8] = 1;
 	trau_bits[2 * 8] = 0;
 	trau_bits[2 * 8 + 1] = 1;
-	for (i = 3; i < 19; i++)
+	for (i = 3; i < 20; i++)
 		trau_bits[i * 8] = 1;
 	trau_bits[19 * 8 + 7] = 1;
 
@@ -1143,12 +1176,13 @@ static int encode8_data(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 	d_idx += 2;
 	/* D3 .. D8 */
 	memcpy(trau_bits + 2 * 8 + 2, fr->d_bits + d_idx, 6);
-	/* D9 .. D57 + D'1 .. D'57 */
-	for (i = 3; i < 20; i++) {
+	d_idx += 6;
+	/* D9 .. D63 + D'1 .. D'57 */
+	for (i = 3; i < 19; i++) {
 		memcpy(trau_bits + i * 8 + 1, fr->d_bits + d_idx, 7);
 		d_idx += 7;
 	}
-	/* D'58 .. D'62 */
+	/* D'58 .. D'63 */
 	memcpy(trau_bits + 19 * 8 + 1, fr->d_bits + d_idx, 6);
 	d_idx += 6;
 
@@ -1219,9 +1253,21 @@ static int encode8_oam(ubit_t *trau_bits, const struct osmo_trau_frame *fr)
 
 /*! Encode a TRAU frame from its decoded representation to a sequence of unpacked bits.
  *  \param[out] bits caller-allocated buffer for unpacked outpud bits
- *  \param[in] n_bits size of 'bits' oputput buffer in number of unpacked bits
+ *  \param[in] n_bits size of 'bits' output buffer in number of unpacked bits
  *  \param[in] fr decoded representation of TRAU frame to be encoded
- *  \return 0 number of unpacked output bits generated; negative in case of error */
+ *  \return 0 number of unpacked output bits generated; negative in case of error
+ *
+ * This function exhibits a behavioral quirk which users need to be aware of:
+ * for many frame types, the first 5 bits out of user-provided fr->c_bits[]
+ * (corresponding to C1..C5) are ignored and replaced with internally supplied
+ * constant values, where the function "knows" what the correct frame type code
+ * should be.  This behavioral quirk is arguably a design defect, but it cannot
+ * be changed without breaking some existing applications that rely on this
+ * behavior and skip setting those bits themselves.  For TFO applications
+ * where C1..C5 may need to be modified (especially TFO-AMR where this C1..C5
+ * modification must be done before CRC computation), please use
+ * osmo_trau_frame_encode_tfo().
+ */
 int osmo_trau_frame_encode(ubit_t *bits, size_t n_bits, const struct osmo_trau_frame *fr)
 {
 	/* check for sufficient space provided by caller in output buffer */
@@ -1265,11 +1311,11 @@ int osmo_trau_frame_encode(ubit_t *bits, size_t n_bits, const struct osmo_trau_f
 	switch (fr->type) {
 	case OSMO_TRAU16_FT_FR:
 	case OSMO_TRAU16_FT_EFR:
-		return encode16_fr(bits, fr);
+		return encode16_fr(bits, fr, false);
 	case OSMO_TRAU16_FT_HR:
 		return encode16_hr(bits, fr);
 	case OSMO_TRAU16_FT_AMR:
-		return encode16_amr(bits, fr);
+		return encode16_amr(bits, fr, false);
 	case OSMO_TRAU16_FT_OAM:
 		return encode16_oam(bits, fr);
 	case OSMO_TRAU16_FT_IDLE:
@@ -1279,10 +1325,11 @@ int osmo_trau_frame_encode(ubit_t *bits, size_t n_bits, const struct osmo_trau_f
 	case OSMO_TRAU16_FT_DATA:
 		return encode16_data(bits, fr);
 	case OSMO_TRAU16_FT_D145_SYNC:
+		return encode16_d145_sync(bits, fr);
 	case OSMO_TRAU16_FT_EDATA:
 		return encode16_edata(bits, fr);
 	case OSMO_TRAU8_SPEECH:
-		return encode8_hr(bits, fr);
+		return encode8_hr(bits, fr, false);
 	case OSMO_TRAU8_DATA:
 		return encode8_data(bits, fr);
 	case OSMO_TRAU8_OAM:
@@ -1299,6 +1346,63 @@ int osmo_trau_frame_encode(ubit_t *bits, size_t n_bits, const struct osmo_trau_f
 	}
 }
 
+/*! Encode a TFO frame from its decoded representation to a sequence of unpacked bits.
+ *  \param[out] bits caller-allocated buffer for unpacked outpud bits
+ *  \param[in] n_bits size of 'bits' output buffer in number of unpacked bits
+ *  \param[in] fr decoded representation of TRAU frame to be encoded
+ *  \return 0 number of unpacked output bits generated; negative in case of error
+ *
+ * Compared to regular osmo_trau_frame_encode(), this TFO-specific TRAU frame
+ * encoding function restricts the set of possible frame types to those that
+ * are valid for TFO, restricts the direction to TRAU-UL, reduces the output
+ * space requirement (there is no timing alignment extension) and always uses
+ * all fr->c_bits provided by the application.
+ */
+int osmo_trau_frame_encode_tfo(ubit_t *bits, size_t n_bits, const struct osmo_trau_frame *fr)
+{
+	/* TFO frames are TRAU-UL only */
+	if (fr->dir != OSMO_TRAU_DIR_UL)
+		return -EINVAL;
+
+	/* check for sufficient space provided by caller in output buffer */
+	/* there is no timing alignment in TRAU-UL direction! */
+	switch (fr->type) {
+	case OSMO_TRAU16_FT_FR:
+	case OSMO_TRAU16_FT_EFR:
+	case OSMO_TRAU16_FT_AMR:
+		if (n_bits < 1 * 40 * 8)
+			return -ENOSPC;
+		break;
+	case OSMO_TRAU8_SPEECH:
+	case OSMO_TRAU8_AMR_LOW:
+	case OSMO_TRAU8_AMR_6k7:
+	case OSMO_TRAU8_AMR_7k4:
+		if (n_bits < 1 * 20 * 8)
+			return -ENOSPC;
+		break;
+	default:
+		return -EINVAL;
+	}
+
+	switch (fr->type) {
+	case OSMO_TRAU16_FT_FR:
+	case OSMO_TRAU16_FT_EFR:
+		return encode16_fr(bits, fr, true);
+	case OSMO_TRAU16_FT_AMR:
+		return encode16_amr(bits, fr, true);
+	case OSMO_TRAU8_SPEECH:
+		return encode8_hr(bits, fr, true);
+	case OSMO_TRAU8_AMR_LOW:
+		return encode8_amr_low(bits, fr);
+	case OSMO_TRAU8_AMR_6k7:
+		return encode8_amr_67(bits, fr);
+	case OSMO_TRAU8_AMR_7k4:
+		return encode8_amr_74(bits, fr);
+	default:
+		return -EINVAL;
+	}
+}
+
 /*! Decode/parse a 16k TRAU frame from unpacked bits to decoded format.
  *  \param[out] fr caller-allocated output data structure
  *  \param[in] bits unpacked bits containing raw TRAU frame; must be aligned
@@ -1349,13 +1453,56 @@ int osmo_trau_frame_decode_16k(struct osmo_trau_frame *fr, const ubit_t *bits,
 		return decode16_edata(fr, bits, dir);
 	case TRAU_FT_D145_SYNC:
 		fr->type = OSMO_TRAU16_FT_D145_SYNC;
-		return decode16_edata(fr, bits, dir);
+		return decode16_data(fr, bits, dir);
 
 	default:
 		return -EINVAL;
 	}
 }
 
+/*! Decode/parse a 16k TFO frame from unpacked bits to decoded format.
+ *  \param[out] fr caller-allocated output data structure
+ *  \param[in] bits unpacked bits containing raw TFO frame; must be aligned
+ *  \returns 0 in case of success; negative on error
+ *
+ * TFO applications that need to decode TFO frames of TS 28.062 section 5.2.1
+ * (FR/EFR) or section 5.2.2.1 (AMR_TFO_16k) should use this function instead
+ * of regular osmo_trau_frame_decode_16k() in order to get appropriately
+ * modified handling of C1..C5 frame type bits:
+ *
+ * - C5 needs to be ignored because it is EMBED bit in TFO frames;
+ * - For interoperability with other potential TFO-AMR implementations
+ *   out in the world that aren't GSM AMR-FR, we need to recognize additional
+ *   C1..C4 bit patterns as being AMR equivalents: see Table 5.2.2.1-2 in
+ *   3GPP TS 28.062.
+ */
+int osmo_trau_frame_decode_tfo_16k(struct osmo_trau_frame *fr, const ubit_t *bits)
+{
+	uint8_t cbits4 = get_bits(bits, 17, 4);
+	/* We don't look at bit C5 because it is EMBED bit in TFO frames */
+
+	fr->type = OSMO_TRAU_FT_NONE;
+	fr->dir = OSMO_TRAU_DIR_UL;	/* TFO frames are modified TRAU-UL */
+	fr->dl_ta_usec = 0;
+
+	switch (cbits4) {
+	case TRAU_FT_FR_UP >> 1:
+		fr->type = OSMO_TRAU16_FT_FR;
+		return decode16_fr(fr, bits, fr->dir);
+	case TRAU_FT_EFR >> 1:
+		fr->type = OSMO_TRAU16_FT_EFR;
+		return decode16_fr(fr, bits, fr->dir);
+	case TRAU_FT_AMR >> 1:
+	case 0x4:	/* HR_AMR */
+	case 0x6:	/* UMTS_AMR_2 */
+	case 0xB:	/* OHR_AMR */
+		fr->type = OSMO_TRAU16_FT_AMR;
+		return decode16_amr(fr, bits, fr->dir);
+	default:
+		return -EINVAL;
+	}
+}
+
 #define TRAU8_FT_AMR_NO_SPEECH_CMI	0x10	/* 1, 0, 0, 0, 0 */
 #define TRAU8_FT_AMR_NO_SPEECH_CMR	0x14	/* 1, 0, 1, 0, 0 */
 #define TRAU8_FT_AMR_475_515_590	0..7
@@ -1374,7 +1521,7 @@ static bool is_hr(const ubit_t *bits)
 		return false;
 	if (bits[16] != 0 || bits[17] != 1)
 		return false;
-	for (i = 24; i < 20 * 8; i += 16) {
+	for (i = 24; i < 20 * 8; i += 8) {
 		if (bits[i] != 1)
 			return false;
 	}
@@ -1396,7 +1543,7 @@ static bool is_amr_low(const ubit_t *bits)
 		return false;
 	if (bits[24] != 0 || bits[25] != 1)
 		return false;
-	for (i = 32; i < 20 * 8; i += 16) {
+	for (i = 32; i < 20 * 8; i += 8) {
 		if (bits[i] != 1)
 			return false;
 	}
@@ -1431,7 +1578,7 @@ static bool is_amr_67(const ubit_t *bits)
 /*!< check sync pattern for AMR 7.4kBit/s */
 static bool is_amr_74(const ubit_t *bits)
 {
-	if (bits[0] != 0 || bits[1] != 0 || bits[2] != 0)
+	if (bits[0] != 0 || bits[1] != 0 || bits[2] != 1)
 		return false;
 	if (bits[8] != 0)
 		return false;
@@ -1461,32 +1608,107 @@ int osmo_trau_frame_decode_8k(struct osmo_trau_frame *fr, const ubit_t *bits,
 		if (dir == OSMO_TRAU_DIR_UL) {
 			switch (cbits5) {	/* Section 5.2.4.1.1 */
 			case 0x02:
+				fr->type = OSMO_TRAU8_SPEECH;
 				return decode8_hr(fr, bits, dir);
 			case 0x07:
+				fr->type = OSMO_TRAU8_DATA;
 				return decode8_data(fr, bits, dir);
-			case 0x13:
+			case 0x0B:
+				fr->type = OSMO_TRAU8_OAM;
 				return decode8_oam(fr, bits, dir);
 			}
 		} else {
 			/* Downlink */
 			switch (cbits5 >> 2) {	/* Section 5.2.4.1.2 */
 			case 0:
+				fr->type = OSMO_TRAU8_SPEECH;
 				return decode8_hr(fr, bits, dir);
 			case 1:
+				fr->type = OSMO_TRAU8_DATA;
 				return decode8_data(fr, bits, dir);
 			case 2:
+				fr->type = OSMO_TRAU8_OAM;
 				return decode8_oam(fr, bits, dir);
 			}
 		}
 	} else if (is_amr_low(bits)) {
+		fr->type = OSMO_TRAU8_AMR_LOW;
 		return decode8_amr_low(fr, bits, dir);
 	} else if (is_amr_67(bits)) {
+		fr->type = OSMO_TRAU8_AMR_6k7;
 		return decode8_amr_67(fr, bits, dir);
 	} else if (is_amr_74(bits)) {
+		fr->type = OSMO_TRAU8_AMR_7k4;
 		return decode8_amr_74(fr, bits, dir);
 	}
 
 	return -EINVAL;
 }
 
+/*! Decode/parse an HRv1 TFO frame from unpacked bits to decoded format.
+ *  \param[out] fr caller-allocated output data structure
+ *  \param[in] bits 160 bits containing the lsb extracted from PCM samples;
+ *  must be aligned
+ *  \returns 0 in case of success; negative on error
+ *
+ * TFO applications that need to decode TFO frames of TS 28.062 section 5.3.1
+ * (GSM_HR codec) should use this function instead of regular
+ * osmo_trau_frame_decode_8k() in order to get these two behavior
+ * modifications:
+ *
+ * - The sync pattern check needs to be skipped because the sync pattern
+ *   may be partially overwritten by embedded TFO messages;
+ * - C5 needs to be ignored because it is EMBED bit in TFO frames.
+ *
+ * In the case of TFO, unlike regular TRAU-UL frames on Abis/Ater, there is
+ * no need to distinguish between HRv1 and AMR-8k frames by their respective
+ * sync patterns: per TS 28.062, these two TFO frame types never occur
+ * in the same bit position in TFO-superimposed PCM speech octets.
+ * Because HRv1 is the only TFO frame type that can occur in the configuration
+ * of TS 28.062 section 5.3 (only the one lsb in each PCM speech octet is
+ * overwritten), this function skips all attempts at frame type classification:
+ * the TFO application is responsible for detecting the presence of valid
+ * TFO frames for the negotiated codec type.
+ */
+int osmo_trau_frame_decode_tfo_hr1(struct osmo_trau_frame *fr, const ubit_t *bits)
+{
+	fr->type = OSMO_TRAU8_SPEECH;
+	fr->dir = OSMO_TRAU_DIR_UL;	/* TFO frames are modified TRAU-UL */
+	fr->dl_ta_usec = 0;
+	return decode8_hr(fr, bits, fr->dir);
+}
+
+/*! Decode/parse an AMR_TFO_8+8k frame from unpacked bits to decoded format.
+ *  \param[out] fr caller-allocated output data structure
+ *  \param[in] bits 160 bits containing the second lsb extracted from
+ *  PCM samples; must be aligned
+ *  \returns 0 in case of success; negative on error
+ *
+ * This function is provided for TFO applications that need to decode
+ * TFO frames of TS 28.062 section 5.2.2.2, format named AMR_TFO_8+8k.
+ * The second lsb extracted from PCM samples is the entity being decoded here.
+ * Compared to osmo_trau_frame_decode_8k(), the present function requires
+ * the direction to be TRAU-UL (as always the case in TFO) and accepts
+ * only AMR-8k frame types.
+ */
+int osmo_trau_frame_decode_tfo_amr_8k(struct osmo_trau_frame *fr, const ubit_t *bits)
+{
+	fr->type = OSMO_TRAU_FT_NONE;
+	fr->dir = OSMO_TRAU_DIR_UL;	/* TFO frames are modified TRAU-UL */
+	fr->dl_ta_usec = 0;
+
+	if (is_amr_low(bits)) {
+		fr->type = OSMO_TRAU8_AMR_LOW;
+		return decode8_amr_low(fr, bits, fr->dir);
+	} else if (is_amr_67(bits)) {
+		fr->type = OSMO_TRAU8_AMR_6k7;
+		return decode8_amr_67(fr, bits, fr->dir);
+	} else if (is_amr_74(bits)) {
+		fr->type = OSMO_TRAU8_AMR_7k4;
+		return decode8_amr_74(fr, bits, fr->dir);
+	}
+
+	return -EINVAL;
+}
+
 /* }@ */
